<?php
/**********************************************************************
    Copyright (C) FrontAccounting, LLC.
	Released under the terms of the GNU General Public License, GPL, 
	as published by the Free Software Foundation, either version 3 
	of the License, or (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
    See the License here <http://www.gnu.org/licenses/gpl-3.0.html>.
***********************************************************************/
/* $Revision: 1.8.2.18 $ */
$page_security = 8;
include_once($path_to_root . "/reporting/includes/ExcelWriterXML.php");
include_once($path_to_root . "/admin/db/company_db.inc");
include_once($path_to_root . "/admin/db/fiscalyears_db.inc");
include_once($path_to_root . "/config.php");
// xml version
class FrontReport extends ExcelWriterXML
{
	var $size;
	var $company;
	var $user;
	var $host;
	var $fiscal_year;
	var $title;
	var $filename;
	var $unique_name;
	var $path;
	var $rtl;
	var $code;
	var $bottomMargin = 0;
	var $lineHeight;
	var $leftMargin = 0;

	var $cols;
	var $params;
	var $headers;
	var $aligns;
	var $headers2;
	var $aligns2;
	var $cols2;
	var $fontSize;
	var $oldFontSize;
	var $currency;
	var $row = 9999999;
	var $y;
	var $numcols;
	var $excelColWidthFactor;
	var $endLine;
	var $headerFunc;
	
	var $sheet;

	function FrontReport($title, $filename, $size = 'A4', $fontsize = 9, 
		$orientation = 'P', $margins = NULL, $excelColWidthFactor = 6.5)
	{
		global $comp_path, $dateseps, $thoseps, $decseps, $page_security;
		
		if (!$_SESSION["wa_current_user"]->can_access_page($page_security))
		{
			display_error(_("The security settings on your account do not permit you to print this report"));
			end_page();
			exit;
		}
		
		$this->size = $size;
		$this->title = $title;
		$this->lineHeight = 12;
		$this->endLine = 760;
		$this->fontSize = $fontsize;
		$this->oldFontSize = 0;
		$this->y = 1;
		$this->currency = '';
		$this->excelColWidthFactor = $excelColWidthFactor;
		$this->rtl = ($_SESSION['language']->dir === 'rtl' ? 'rtl' : 'ltr');
		$this->code = $_SESSION['language']->encoding;
		$this->filename = $filename.".xml";
		$this->unique_name = uniqid('').".xml";
		$this->path = $comp_path.'/'.user_company(). '/pdf_files';
		$this->ExcelWriterXML($this->filename);
		$this->setCharSet($this->code);
		$this->overwriteFile();
		$this->showErrorSheet(true);
		
		$this->sheet =& $this->addSheet($this->worksheetNameGenerator($this->title));
		if ($this->rtl === 'rtl')
			$this->sheet->displayRightToLeft();
		$formatTitle =& $this->addStyle('formatTitle');	
		$formatTitle->fontSize('16');
		$formatTitle->fontBold();
		$formatTitle->border('Top', '2', 'darkgray');
	
		$how = user_date_format();
		$sep = $dateseps[user_date_sep()];
		if ($sep == '.')
			$sep = "\\.";
		if ($how == 0)
		{
			$dateformat_long = "mm{$sep}dd{$sep}yyyy\ \ hh:mm\ am/pm";
			$dateformat = "mm{$sep}dd{$sep}yyyy";
		}	
		elseif ($how == 1)	
		{
			$dateformat_long = "dd{$sep}mm{$sep}yyyy\ \ hh:mm";
			$dateformat = "dd{$sep}mm{$sep}yyyy";
		}	
		else	
		{
			$dateformat_long = "yyyy{$sep}mm{$sep}dd\ \ hh:mm";
			$dateformat = "yyyy{$sep}mm{$sep}dd";
		}	
		$formatDateTime =& $this->addStyle('formatDateTime');
		$formatDateTime->numberFormatDateTime($dateformat_long);
		$formatDateTime->alignHorizontal('Left');
		$formatDate =& $this->addStyle('formatDate');
		$formatDate->numberFormatDateTime($dateformat);
		$formatDate->alignHorizontal('Left');
		$formatRight =& $this->addStyle('formatRight');
		$formatRight->alignHorizontal('Right');
		
		$formatHeaderLeft =& $this->addStyle("formatHeaderLeft");
		$formatHeaderLeft->fontItalic();
		$formatHeaderLeft->border('Top', '2', 'darkgray');
		$formatHeaderLeft->border('Bottom', '2', 'darkgray');
		$formatHeaderLeft->alignVertical('Center');

		$formatTopHeaderLeft =& $this->addStyle("formatTopHeaderLeft");
		$formatTopHeaderLeft->fontItalic();
		$formatTopHeaderLeft->border('Top', '2', 'darkgray');
		$formatTopHeaderLeft->border('Bottom', '2', 'darkgray');
		$formatTopHeaderLeft->alignVertical('Center');

		$formatBottomHeaderLeft =& $this->addStyle("formatBottomHeaderLeft");
		$formatBottomHeaderLeft->fontItalic();
		$formatBottomHeaderLeft->border('Top', '2', 'darkgray');
		$formatBottomHeaderLeft->border('Bottom', '2', 'darkgray');
		$formatBottomHeaderLeft->alignVertical('Center');

		$formatHeaderRight =& $this->addStyle("formatHeaderRight");
		$formatHeaderRight->fontItalic();
		$formatHeaderRight->alignHorizontal('Right');
		$formatHeaderRight->border('Top', '2', 'darkgray');
		$formatHeaderRight->border('Bottom', '2', 'darkgray');
		$formatHeaderRight->alignVertical('Center');

		$formatTopHeaderRight =& $this->addStyle("formatTopHeaderRight");
		$formatTopHeaderRight->fontItalic();
		$formatTopHeaderRight->alignHorizontal('Right');
		$formatTopHeaderRight->border('Top', '2', 'darkgray');
		$formatTopHeaderRight->border('Bottom', '2', 'darkgray');
		$formatTopHeaderRight->alignVertical('Center');

		$formatBottomHeaderRight =& $this->addStyle("formatBottomHeaderRight");
		$formatBottomHeaderRight->fontItalic();
		$formatBottomHeaderRight->alignHorizontal('Right');
		$formatBottomHeaderRight->border('Top', '2', 'darkgray');
		$formatBottomHeaderRight->border('Bottom', '2', 'darkgray');
		$formatBottomHeaderRight->alignVertical('Center');
		
		$formatFooter =& $this->addStyle("formatFooter");
		$formatFooter->border('Top', '2', 'darkgray');
		$this->SetHeaderType("header");
	}
	
	/*
		Set header handler
	*/
	function SetHeaderType($name) {
		$this->headerFunc = $name;
	}
	// Check a given name to see if it's a valid Excel worksheet name,
	// and fix if necessary
	function worksheetNameGenerator($name)
	{
		// First, strip out characters which aren't allowed
		$illegal_chars = array(':', '\\', '/', '?', '*', '[', ']');
		for ($i = 0; $i < count($illegal_chars); $i++)
			$name = str_replace($illegal_chars[$i], '', $name);
		// Now, if name is longer than 31 chars, truncate it
		if (strlen($name) > 31)
			$name = substr($name, 0, 31);
		return $name;
	}
 	
	function NewPage()
	{
		if (method_exists($this, $this->headerFunc))	// draw predefined page layout if any
			$this->{$this->headerFunc}();
	}
	
	function NumFormat($dec) 
	{
		global $thoseps,$decseps;
		
		$dec = (int)$dec;
		$stylename = 'formatAmount'.$dec;
		if (!isset($this->styles[$stylename])) 
		{
			$tsep = $thoseps[user_tho_sep()];
			$dsep = $decseps[user_dec_sep()];
			$format = "###{$tsep}###{$tsep}###{$tsep}##0";
			if ($dec>0)
				$format .= "{$dsep}".str_repeat('0',$dec);
			$style =& $this->addStyle($stylename);
			$style->numberFormat($format);
			$style->alignHorizontal('Right');
		}
		return $stylename;
	}

	function Font($fontname = '', $style = 'normal')
	{
	}

	function Info($params, $cols, $headers, $aligns,
		$cols2 = null, $headers2 = null, $aligns2 = null)
	{
		global $app_title, $version, $power_by, $power_url;
		$this->company = get_company_prefs();
		$this->docTitle($this->title);
		$this->docAuthor($app_title . ' ' . $version);
		$this->docCompany($this->company['coy_name']);
		$this->docManager($power_by . ' - ' . $power_url);
		$year = get_current_fiscalyear();
		if ($year['closed'] == 0)
			$how = _("Active");
		else
			$how = _("Closed");
		$this->fiscal_year = sql2date($year['begin']) . " - " . sql2date($year['end']) . "  " . "(" . $how . ")";
		$this->user = $_SESSION["wa_current_user"]->name;
		$this->host = $_SERVER['SERVER_NAME'];
		$this->params = $params;
		$this->cols = $cols;
		$this->headers = $headers;
		$this->aligns = $aligns;
		$this->cols2 = $cols2;
		$this->headers2 = $headers2;
		$this->aligns2 = $aligns2;
		$this->numcols = count($this->headers);
		$tcols = count($this->headers2);
		if ($tcols > $this->numcols)
			$this->numcols = $tcols;
		for ($i = 0; $i < $this->numcols; $i++)
			$this->sheet->columnWidth($i+1, ($this->cols[$i + 1] - $this->cols[$i]) / 1.1);
	}

	function Header()
	{
		$this->y = 0;
		$tcol = $this->numcols - 1;
		$this->NewLine();
		$this->sheet->rowHeight($this->y, 20);
		$this->sheet->writeString($this->y, 1, $this->title, 'formatTitle');
		$this->sheet->cellMerge($this->y, 1, $tcol, 0);

		$this->NewLine();

		$str = _("Print Out Date") . ':';
		$this->sheet->writeString($this->y, 1, $str);
		$date = $this->sheet->convertMysqlDateTime(date('Y-m-d H:i:s'));
		$this->sheet->writeDateTime($this->y, 2, $date, 'formatDateTime');
		$this->sheet->writeString($this->y, $tcol, $this->company['coy_name']);
		$this->sheet->cellMerge($this->y, $tcol, 1, 0);
		$this->NewLine();
		$str = _("Fiscal Year") . ':';
		$this->sheet->writeString($this->y, 1, $str);
		$str = $this->fiscal_year;
		$this->sheet->writeString($this->y, 2, $str);
		$this->sheet->writeString($this->y, $tcol, $this->host);
		$this->sheet->cellMerge($this->y, $tcol, 1, 0);
		for ($i = 1; $i < count($this->params); $i++)
		{
			if ($this->params[$i]['from'] != '')
			{
				$this->NewLine();
				$str = $this->params[$i]['text'] . ':';
				$this->sheet->writeString($this->y, 1, $str);
				$str = $this->params[$i]['from'];
				if ($this->params[$i]['to'] != '')
					$str .= " - " . $this->params[$i]['to'];
				$this->sheet->writeString($this->y, 2, $str);
				if ($i == 1)
				{
					$this->sheet->writeString($this->y, $tcol, $this->user);
					$this->sheet->cellMerge($this->y, $tcol, 1, 0);
				}	
			}
		}
		if ($this->params[0] != '') // Comments
		{
			$this->NewLine();
			$str = _("Comments") . ':';
			$this->sheet->writeString($this->y, 1, $str);
			$this->sheet->writeString($this->y, 2, $this->params[0]);
		}
		$this->NewLine();
		if ($this->headers2 != null)
		{
			for ($i = 0, $j = 0; $i < $this->numcols; $i++)
			{
				if ($this->cols2[$j] >= $this->cols[$i] && $this->cols2[$j] <= $this->cols[$i + 1])
				{
					if ($this->aligns2[$j] == "right")
						$this->sheet->writeString($this->y, $i + 1, $this->headers2[$j], 'formatHeaderRight');
					else	
						$this->sheet->writeString($this->y, $i + 1, $this->headers2[$j], 'formatHeaderLeft');
					$j++;	
				}
				else
					$this->sheet->writeString($this->y, $i + 1, "", 'formatHeaderLeft');
			}		
			$this->NewLine();
		}

		for ($i = 0; $i < $this->numcols; $i++)
		{
			if (!isset($this->headers[$i]))
				$header = "";
			else
				$header = $this->headers[$i];
			if ($this->aligns[$i] == "right")
				$this->sheet->writeString($this->y, $i + 1, $header, 'formatHeaderRight');
			else	
				$this->sheet->writeString($this->y, $i + 1, $header, 'formatHeaderLeft');
		}		
		$this->NewLine();
	}

	function Header2($myrow, $branch, $sales_order, $bankaccount, $doctype)
	{
		return;
	}

	// Alternate header style - primary differences are for PDFs
	function Header3()
	{
		// Flag to make sure we only print the company name once
		$companyNamePrinted = false;
		
		$this->y = 0;
		$tcol = $this->numcols - 1;
		$this->NewLine();
		$this->sheet->rowHeight($this->y, 20);
		$this->sheet->writeString($this->y, 1, $this->title, 'formatTitle');
		$this->sheet->cellMerge($this->y, 1, $tcol, 0);
		
		$this->NewLine();

		// Dimension 1 - optional
		// - only print if available and not blank
		if (count($this->params) > 3)
			if ($this->params[3]['from'] != '')
			{
				$this->NewLine();
				$str = $this->params[3]['text'] . ':';
				$this->sheet->writeString($this->y, 1, $str, 'formatLeft');
				$this->sheet->writeString($this->y, 2, $this->params[3]['from'], 'formatLeft');
				// Company Name - at end of this row
				if (!$companyNamePrinted)
				{
					$this->sheet->writeString($this->y, $tcol, $this->company['coy_name'], 'formatLeft');
					$this->sheet->cellMerge($this->y, $tcol, 1, 0);
					$companyNamePrinted = true;
				}
			}


		// Dimension 2 - optional
		// - only print if available and not blank
		if (count($this->params) > 4)
			if ($this->params[4]['from'] != '')
			{
				$this->NewLine();
				$str = $this->params[4]['text'] . ':';
				$this->sheet->writeString($this->y, 1, $str, 'formatLeft');
				$this->sheet->writeString($this->y, 2, $this->params[4]['from'], 'formatLeft');
				// Company Name - at end of this row
				if (!$companyNamePrinted)
				{
					$this->sheet->writeString($this->y, $tcol, $this->company['coy_name'], 'formatLeft');
					$this->sheet->CellMerge($this->y, $tcol, 1, 0);
					$companyNamePrinted = true;
				}
			}
			
		// Tags - optional
		// TBD!!!		
	
		// Report Date - time period covered
		// - can specify a range, or just the end date (and the report contents
		//   should make it obvious what the beginning date is)
		$this->NewLine();
		$str = _("Report Date") . ':';
		$this->sheet->writeString($this->y, 1, $str, 'formatLeft');
		$str = '';
		if ($this->params[1]['from'] != '')
			$str = $this->params[1]['from'] . ' - ';
		$str .= $this->params[1]['to'];
		$this->sheet->writeString($this->y, 2, $str, 'formatLeft');
		// Company Name - at end of this row
		if (!$companyNamePrinted)
		{
			$this->sheet->writeString($this->y, $tcol, $this->company['coy_name'], 'formatLeft');
			$this->sheet->cellMerge($this->y, $tcol, 1, 0);
			$companyNamePrinted = true;
		}
			
		// Timestamp of when this copy of the report was generated 
		$this->NewLine();
		$str = _("Generated At") . ':';
		$this->sheet->writeString($this->y, 1, $str, 'formatLeft');
		$this->sheet->writeString($this->y, 2, Today() . "  ".Now(), 'formatLeft');

		// Name of the user that generated this copy of the report
		$this->NewLine();
		$str = _("Generated By") . ':';
		$this->sheet->writeString($this->y, 1, $str, 'formatLeft');
		$str = $this->user;
		$this->sheet->writeString($this->y, 2, $str, 'formatLeft');

		// Comments - display any user-generated comments for this copy of the report
		if ($this->params[0] != '')
		{
			$this->NewLine();
			$str = _("Comments") . ':';
			$this->sheet->writeString($this->y, 1, $str, 'formatLeft');
			$this->sheet->writeString($this->y, 2, $this->params[0], 'formatLeft');
		}
		$this->NewLine();

		if ($this->headers2 != null)
		{
			for ($i = 0, $j = 0; $i < $this->numcols; $i++)
			{
				if ($this->cols2[$j] >= $this->cols[$i] && $this->cols2[$j] <= $this->cols[$i + 1])
				{
					if ($this->aligns2[$j] == "right")
						$this->sheet->writeString($this->y, $i+1, $this->headers2[$j], 'formatTopHeaderRight');
					else	
						$this->sheet->writeString($this->y, $i+1, $this->headers2[$j], 'formatTopHeaderLeft');
					$j++;	
				}
				else
					$this->sheet->writeString($this->y, $i+1, "", 'formatTopHeaderLeft');
			}		
			$this->NewLine();
		}

		for ($i = 0; $i < $this->numcols; $i++)
		{
			if (!isset($this->headers[$i]))
				$header = "";
			else
				$header = $this->headers[$i];
			if ($this->aligns[$i] == "right")
				if ($this->headers2 == null)
					$this->sheet->writeString($this->y, $i+1, $header, 'formatHeaderRight');
				else
					$this->sheet->writeString($this->y, $i+1, $header, 'formatBottomHeaderRight');
			else	
				if ($this->headers2 == null)
					$this->sheet->writeString($this->y, $i+1, $header, 'formatHeaderLeft');
				else
					$this->sheet->writeString($this->y, $i+1, $header, 'formatBottomHeaderLeft');
		}
		$this->NewLine();
	}

	/**
	 * Format a numeric string date into something nicer looking.
	 *
	 * @param string $date Date string to be formatted.
	 * @param int $input_format Format of the input string.  Possible values are:<ul><li>0: user's default (default)</li></ul>
	 * @param int $output_format Format of the output string.  Possible values are:<ul><li>0: Month (word) Day (numeric), 4-digit Year - Example: January 1, 2000 (default)</li><li>1: Month 4-digit Year - Example: January 2000</li><li>2: Month Abbreviation 4-digit Year - Example: Jan 2000</li></ul>
	 * @access public
	 */
	function DatePrettyPrint($date, $input_format = 0, $output_format = 0)
	{
		if ($date != '')
		{
			$date = date2sql($date);
			$year = (int) (substr($date, 0, 4));
			$month = (int) (substr($date, 5, 2));
			$day = (int) (substr($date, 8, 2));
			if ($output_format == 0)
				return(date('F j, Y', mktime(12, 0, 0, $month, $day, $year)));
			elseif ($output_format == 1)
				return(date('F Y', mktime(12, 0, 0, $month, $day, $year)));
			elseif ($output_format == 2)
				return(date('M Y', mktime(12, 0, 0, $month, $day, $year)));
		}
		else
			return $date;
	}

	function AddImage($logo, $x, $y, $w, $h)
	{
		return;
	}

	function SetDrawColor($r, $g, $b)
	{
		return;
	}

	function SetTextColor($r, $g, $b)
	{
		return;
	}

	function SetFillColor($r, $g, $b)
	{
		return;
	}
	
	function GetCellPadding()
	{
		return 0;
	}
	
	function SetCellPadding($pad)
	{
		return;
	}
	
	function Text($c, $txt, $n=0, $corr=0, $r=0, $align='left', $border=0, $fill=0, $link=NULL, $stretch=0)
	{
		return;
	}

	function TextWrap($xpos, $ypos, $len, $str, $align = 'left', $border = 0, $fill = 0, $link = NULL, $stretch = 0)
	{
		return;
	}

	function TextCol($c, $n, $txt, $corr=0, $r=0, $border=0, $fill=0, $link=NULL, $stretch=0)
	{
		if ($this->aligns[$c] == 'right')
			$this->sheet->writeString($this->y, $c + 1, $txt, 'formatRight');
		else	
			$this->sheet->writeString($this->y, $c + 1, $txt);
		if ($n - $c > 1)
			$this->sheet->cellMerge($this->y, $c + 1, $n - $c - 1, 0);
	}

	function AmountCol($c, $n, $txt, $dec=0, $corr=0, $r=0, $border=0, $fill=0, $link=NULL, $stretch=0, $color_red=false) 
	{ 
		if (!is_numeric($txt))
			$txt = 0;
		$this->sheet->writeNumber($this->y, $c + 1, $txt, $this->NumFormat($dec)); 
	}
	
	function AmountCol2($c, $n, $txt, $dec=0, $corr=0, $r=0, $border=0, $fill=0, $link=NULL, $stretch=0, $color_red=false, $amount_locale = NULL, $amount_format = NULL) 
	{ 
		if (!is_numeric($txt))
			$txt = 0;
		$this->sheet->writeNumber($this->y, $c + 1, $txt, $this->NumFormat($dec)); 
	}

	function DateCol($c, $n, $txt, $conv=false, $corr=0, $r=0, $border=0, $fill=0, $link=NULL, $stretch=0) 
	{
		if (!$conv)
			$txt = date2sql($txt);
		$date = $this->sheet->convertMysqlDate($txt);
		$this->sheet->writeDateTime($this->y, $c + 1, $date, 'formatDate');
	}

	function TextCol2($c, $n, $txt, $corr=0, $r=0, $border=0, $fill=0, $link=NULL, $stretch=0)
	{
		$this->sheet->writeString($this->y, $c + 1, $txt);
		if ($n - $c > 1)
			$this->sheet->cellMerge($this->y, $c + 1, $n - $c - 1, 0);
	}

	function TextColLines($c, $n, $txt, $corr=0, $r=0, $border=0, $fill=0, $link=NULL, $stretch=0)
	{
		return;
	}

	function TextWrapLines($c, $width, $txt, $align='left', $border=0, $fill=0, $link=NULL, $stretch=0)
	{
		return;
	}

	/**
	 * Crude text wrap calculator based on PDF version.
	 */
	function TextWrapCalc($txt, $width, $spacebreak=false)
	{
		// Assume an average character width
		$avg_char_width = 5;
		$ret = "";
		$txt2 = $txt;
		$w = strlen($txt) * $avg_char_width;
		if ($w > $width && $w > 0 && $width != 0)
		{
			$n = strlen($txt);
			$k = intval($n * $width / $w);
			if ($k > 0 && $k < $n)
			{
				$txt2 = substr($txt, 0, $k);
				if ($spacebreak && (($pos = strrpos($txt2, " ")) !== false))
				{
					$txt2 = substr($txt2, 0, $pos);
					$ret = substr($txt, $pos+1);
				}
				else
					$ret = substr($txt, $k);
			}
		}
		return array($txt2, $ret);
	}

	function SetLineStyle($style)
	{
		return;
	}

	function SetLineWidth($width)
	{
		return;
	}
	
	function LineTo($from, $row, $to, $row2)
	{
		return;
	}

	function Line($row, $height = 0)
	{
		return;
	}

	function UnderlineCell($c, $r = 0, $type = 1, $linewidth = 0, $style = array())
	{
		return;
	}

	function NewLine($l=1, $np=0, $h=NULL)
	{
		$this->y += $l;
	}

	function End($email=0, $subject=null, $myrow=null, $doctype = 0)
	{
		++$this->y;
		$this->sheet->cellMerge($this->y, 1, $this->numcols - 1, 0);
		$this->sheet->writeString($this->y, 1, "", 'formatFooter');
		$this->writeData($this->path."/".$this->unique_name);
		// first have a look through the directory, 
		// and remove old temporary pdfs
		if ($d = @opendir($this->path)) {
			while (($file = readdir($d)) !== false) {
				if (!is_file($this->path.'/'.$file) || $file == 'index.php') continue;
				// then check to see if this one is too old
				$ftime = filemtime($this->path.'/'.$file);
				 // seems 3 min is enough for any report download, isn't it?
				if (time()-$ftime > 180){
					unlink($this->path.'/'.$file);
				}
			}
			closedir($d);
		}
		meta_forward($_SERVER['PHP_SELF'], "xml=1&filename=$this->filename&unique=$this->unique_name");
		exit();
		// first have a look through the directory, 
		// and remove old temporary pdfs
		/*
		if ($d = @opendir($dir)) {
			while (($file = readdir($d)) !== false) {
				if (!is_file($dir.'/'.$file) || $file == 'index.php') continue;
			// then check to see if this one is too old
				$ftime = filemtime($dir.'/'.$file);
			 // seems 3 min is enough for any report download, isn't it?
				if (time()-$ftime > 180){
					unlink($dir.'/'.$file);
				}
			}
			closedir($d);
		}
		*/
	}
}

?>